@@ -99,6 +99,69 @@ class CoerciveIntEnum(IntEnum):
         raise TypeError(f"{value} cannot convert to a {cls.__name__}")
 
 
+class CoerciveIntFlag(IntFlag):
+    """
+    Enumerated constants that can be combined using the bitwise operators,
+    with a helper to coerce strings and integers into enumeration members.
+    """
+
+    @classmethod
+    def coerce(cls, value):
+        """
+                Attempt to coerce `value` into a member of this enumeration.
+        <<<<<<< HEAD
+        =======
+
+        >>>>>>> Handling page breaks in tables
+                If value is already a member of this enumeration it is returned unchanged.
+                Otherwise, if it is a string, attempt to convert it (case insensitively, by
+                upcasing) as an enumeration name. Otherwise, if it is an int, attempt to
+                convert it as an enumeration value.
+        <<<<<<< HEAD
+                Otherwise, an exception is raised.
+                Args:
+                    value (IntEnum, str, int): the value to be coerced.
+        =======
+
+                Otherwise, an exception is raised.
+
+                Args:
+                    value (IntEnum, str, int): the value to be coerced.
+
+        >>>>>>> Handling page breaks in tables
+                Raises:
+                    ValueError: if `value` is an int but not a member of this enumeration.
+                    ValueError: if `value` is a string but not a member by name.
+                    TypeError: if `value`'s type is neither a member of the enumeration nor an
+                        int or a string.
+        """
+        if isinstance(value, cls):
+            return value
+
+        if isinstance(value, str):
+            try:
+                flags = cls[value[0].upper()]
+                for char in value[1:]:
+                    flags = flags | cls[char.upper()]
+                return flags
+            except KeyError:
+                raise ValueError(f"{value} is not a valid {cls.__name__}") from None
+
+        if isinstance(value, int):
+            return cls(value)
+
+        raise TypeError(f"{value} cannot convert to a {cls.__name__}")
+
+
+class WrapMode(CoerciveEnum):
+    "Defines how to break and wrap lines in multi-line text."
+    WORD = intern("WORD")
+    "Wrap by word"
+
+    CHAR = intern("CHAR")
+    "Wrap by character"
+
+
 class CharVPos(CoerciveEnum):
     "Defines the vertical position of text relative to the line."
     SUP = intern("SUP")
@@ -142,6 +205,83 @@ class Align(CoerciveEnum):
         return super(cls, cls).coerce(value)
 
 
+class TextEmphasis(CoerciveIntFlag):
+    """
+    Indicates use of bold / italics / underline.
+    This enum values can be combined with & and | operators:
+        style = B | I
+    """
+
+    B = 1
+    "Bold"
+
+    I = 2
+    "Italics"
+
+    U = 4
+    "Underline"
+
+    @property
+    def style(self):
+        return "".join(
+            name for name, value in self.__class__.__members__.items() if value & self
+        )
+
+    @classmethod
+    def coerce(cls, value):
+        if isinstance(value, str):
+            if value == "":
+                return 0
+            if value.upper() == "BOLD":
+                return cls.B
+            if value.upper() == "ITALICS":
+                return cls.I
+            if value.upper() == "UNDERLINE":
+                return cls.U
+        return super(cls, cls).coerce(value)
+
+
+class TableBordersLayout(CoerciveEnum):
+    "Defines how to render table borders"
+
+    ALL = intern("ALL")
+    "Draw all table cells borders"
+
+    NONE = intern("NONE")
+    "Draw zero cells border"
+
+    INTERNAL = intern("INTERNAL")
+    "Draw only internal horizontal & vertical borders"
+
+    MINIMAL = intern("MINIMAL")
+    "Draw only the top horizontal border, below the headings, and internal vertical borders"
+
+    HORIZONTAL_LINES = intern("HORIZONTAL_LINES")
+    "Draw only horizontal lines"
+
+    NO_HORIZONTAL_LINES = intern("NO_HORIZONTAL_LINES")
+    "Draw all cells border except horizontal lines, after the headings"
+
+    SINGLE_TOP_LINE = intern("SINGLE_TOP_LINE")
+    "Draw only the top horizontal border, below the headings"
+
+
+class TableCellFillMode(CoerciveEnum):
+    "Defines which table cells to fill"
+
+    NONE = intern("NONE")
+    "Fill zero table cell"
+
+    ALL = intern("ALL")
+    "Fill all table cells"
+
+    ROWS = intern("ROWS")
+    "Fill only table cells in odd rows"
+
+    COLUMNS = intern("COLUMNS")
+    "Fill only table cells in odd columns"
+
+
 class RenderStyle(CoerciveEnum):
     "Defines how to render shapes"
 
@@ -605,7 +745,8 @@ class Corner(CoerciveEnum):
 
 class FontDescriptorFlags(Flag):
     """An enumeration of the flags for the unsigned 32-bit integer entry in the font descriptor specifying various
-    characteristics of the font. Bit positions are numbered from 1 (low-order) to 32 (high-order)."""
+    characteristics of the font. Bit positions are numbered from 1 (low-order) to 32 (high-order).
+    """
 
     FIXED_PITCH = 0x0000001
     """
@@ -679,7 +820,3 @@ class EncryptionMethod(Enum):
     NO_ENCRYPTION = 0
     RC4 = 1
     AES_128 = 2
-
-
-# This enum is only used internally:
-__pdoc__ = {"DocumentState": False}
