@@ -63,6 +63,7 @@ import re, zlib
 from abc import ABC
 from binascii import hexlify
 from codecs import BOM_UTF16_BE
+from datetime import datetime, timezone
 
 
 def clear_empty_fields(d):
@@ -127,7 +128,7 @@ class Name(str):
         b"[^" + bytes(v for v in range(33, 127) if v not in b"()<>[]{}/%#\\") + b"]"
     )
 
-    def serialize(self) -> str:
+    def serialize(self, _security_handler=None, _obj_id=None) -> str:
         escaped = self.NAME_ESC.sub(
             lambda m: b"#%02X" % m[0][0], self.encode()
         ).decode()
@@ -167,9 +168,7 @@ class PDFObject:
         output.append(f"{self.id} 0 obj")
         output.append("<<")
         if not obj_dict:
-            obj_dict = self._build_obj_dict()
-        if _security_handler:
-            obj_dict = self._encrypt_obj_dict(obj_dict, _security_handler)
+            obj_dict = self._build_obj_dict(_security_handler)
         output.append(create_dictionary_string(obj_dict, open_dict="", close_dict=""))
         output.append(">>")
         content_stream = self.content_stream()
@@ -179,29 +178,21 @@ class PDFObject:
         return "\n".join(output)
 
     def content_stream(self):
-        "Subclass can override this method to indicate the presence of a content stream"
+        "Subclasses can override this method to indicate the presence of a content stream"
         return b""
 
-    def _build_obj_dict(self):
+    def _build_obj_dict(self, security_handler=None):
         """
         Build the PDF Object associative map to serialize,
         based on this class instance properties.
         The property names are converted from snake_case to CamelCase,
         and prefixed with a slash character "/".
         """
-        return build_obj_dict({key: getattr(self, key) for key in dir(self)})
-
-    def _encrypt_obj_dict(self, obj_dict, security_handler):
-        """Encrypt the strings present in the object dictionary"""
-        for key in obj_dict:
-            string = obj_dict[key]
-            if (
-                isinstance(string, str)
-                and string.startswith("(")
-                and string.endswith(")")
-            ):
-                obj_dict[key] = security_handler.encrypt(string[1:-1], self.id)
-        return obj_dict
+        return build_obj_dict(
+            {key: getattr(self, key) for key in dir(self)},
+            _security_handler=security_handler,
+            _obj_id=self.id,
+        )
 
 
 class PDFContentStream(PDFObject):
@@ -215,12 +206,18 @@ class PDFContentStream(PDFObject):
     def content_stream(self):
         return self._contents
 
-    def encrypt(self, security_handler):
-        self._contents = security_handler.encrypt(self._contents, self.id)
-        self.length = len(self._contents)
+    # method override
+    def serialize(self, obj_dict=None, _security_handler=None):
+        if _security_handler:
+            assert not obj_dict
+            if not isinstance(self._contents, (bytearray, bytes)):
+                self._contents = self._contents.encode("latin-1")
+            self._contents = _security_handler.encrypt(self._contents, self.id)
+            self.length = len(self._contents)
+        return super().serialize(obj_dict, _security_handler)
 
 
-def build_obj_dict(key_values):
+def build_obj_dict(key_values, _security_handler=None, _obj_id=None):
     """
     Build the PDF Object associative map to serialize, based on a key-values dict.
     The property names are converted from snake_case to CamelCase,
@@ -241,7 +238,9 @@ def build_obj_dict(key_values):
             value = value.ref
         elif hasattr(value, "serialize"):
             # e.g. PDFArray, PDFString, Name, Destination, Action...
-            value = value.serialize()
+            value = value.serialize(
+                _security_handler=_security_handler, _obj_id=_obj_id
+            )
         elif isinstance(value, bool):
             value = str(value).lower()
         obj_dict[f"/{camel_case(key)}"] = value
@@ -259,7 +258,26 @@ class PDFString(str):
     but then there can be a risk of badly encoding some unicode strings - cf. issue #458
     """
 
-    def serialize(self):
+    def __new__(cls, content, encrypt=False):
+        """
+        Args:
+            content (str): text
+            encrypt (bool): if document encryption is enabled, should this string be encrypted?
+        """
+        self = super().__new__(cls, content)
+        self.encrypt = encrypt
+        return self
+
+    def serialize(self, _security_handler=None, _obj_id=None):
+        if _security_handler and self.encrypt:
+            assert _obj_id
+            return _security_handler.encrypt_string(self, _obj_id)
+        try:
+            self.encode("ascii")
+            # => this string only contains ASCII characters, no need for special encoding:
+            return f"({self})"
+        except UnicodeEncodeError:
+            pass
         if self.USE_HEX_ENCODING:
             # Using the "Hexadecimal String" format defined in the PDF spec:
             hex_str = hexlify(BOM_UTF16_BE + self.encode("utf-16-be")).decode("latin-1")
@@ -267,15 +285,50 @@ class PDFString(str):
         return f'({self.encode("UTF-16").decode("latin-1")})'
 
 
+class PDFDate:
+    def __init__(self, date: datetime, with_tz=False, encrypt=False):
+        """
+        Args:
+            date (datetime): self-explanatory
+            with_tz (bool): should the timezone be encoded in included in the date?
+            encrypt (bool): if document encryption is enabled, should this string be encrypted?
+        """
+        self.date = date
+        self.with_tz = with_tz
+        self.encrypt = encrypt
+
+    def __repr__(self):
+        return f"PDFDate({self.date}, with_tz={self.with_tz}, encrypt={self.encrypt})"
+
+    def serialize(self, _security_handler=None, _obj_id=None):
+        if self.with_tz:
+            assert self.date.tzinfo
+            if self.date.tzinfo == timezone.utc:
+                out_str = f"D:{self.date:%Y%m%d%H%M%SZ%H'%M'}"
+            else:
+                out_str = f"D:{self.date:%Y%m%d%H%M%S%z}"
+                out_str = out_str[:-2] + "'" + out_str[-2:] + "'"
+        else:
+            out_str = f"D:{self.date:%Y%m%d%H%M%S}"
+        if _security_handler and self.encrypt:
+            assert _obj_id
+            return _security_handler.encrypt_string(out_str, _obj_id)
+        return f"({out_str})"
+
+
 class PDFArray(list):
-    def serialize(self):
+    def serialize(self, _security_handler=None, _obj_id=None):
         if all(isinstance(elem, str) for elem in self):
             serialized_elems = " ".join(self)
         elif all(isinstance(elem, int) for elem in self):
             serialized_elems = " ".join(map(str, self))
         else:
             serialized_elems = "\n".join(
-                elem.ref if isinstance(elem, PDFObject) else elem.serialize()
+                elem.ref
+                if isinstance(elem, PDFObject)
+                else elem.serialize(
+                    _security_handler=_security_handler, _obj_id=_obj_id
+                )
                 for elem in self
             )
         return f"[{serialized_elems}]"
@@ -283,7 +336,7 @@ class PDFArray(list):
 
 # cf. section 8.2.1 "Destinations" of the 2006 PDF spec 1.7:
 class Destination(ABC):
-    def serialize(self):
+    def serialize(self, _security_handler=None, _obj_id=None):
         raise NotImplementedError
 
 
@@ -306,7 +359,7 @@ class DestinationXYZ(Destination):
     def __repr__(self):
         return f'DestinationXYZ(page_number={self.page_number}, top={self.top}, left={self.left}, zoom="{self.zoom}", page_ref={self.page_ref})'
 
-    def serialize(self):
+    def serialize(self, _security_handler=None, _obj_id=None):
         left = round(self.left, 2) if isinstance(self.left, float) else self.left
         top = round(self.top, 2) if isinstance(self.top, float) else self.top
         assert self.page_ref
